/*
 * misc.c
 *
 * SELinux hooks related to misc features
 *
 * Copyright (C) 2006-2010, NEC Corporation
 *                          KaiGai Kohei <kaigai@ak.jp.nec.com>
 */
#include "postgres.h"

#include "libpq/libpq-be.h"
#include "miscadmin.h"
#include "sepgsql/hooks.h"
#include "sepgsql/sepgsql.h"
#include "utils/builtins.h"
#include "utils/fmgroids.h"
#include "utils/guc.h"

#include <selinux/selinux.h>

/*
 * sepgsql_client_label
 *
 * security context of the peer process
 */
static char *sepgsql_client_label = NULL;

char *
sepgsql_get_client_label(void)
{
	return sepgsql_client_label;
}

char *
sepgsql_set_client_label(char *new_label)
{
	char   *old_label = sepgsql_client_label;

	sepgsql_client_label = new_label;

#ifdef HAVE_SELINUX
	if (sepgsql_is_enabled())
	{
		/*
		 * NOTE: When an error happened prior to initialization of
		 * SE-PostgreSQL, NULL maight be delivered. In this case,
		 * we don't need to reset avc.
		 */
		if (new_label != NULL)
			sepgsql_avc_switch(new_label);
	}
#endif
	return old_label;
}

void
sepgsql_post_bootstraping(void)
{
#ifdef HAVE_SELINUX
	if (sepgsql_is_enabled())
		sepgsql_initial_labeling();
#endif
}

void
sepgsql_initialize(void)
{
#ifdef HAVE_SELINUX
	if (sepgsql_is_enabled())
	{
		security_context_t	context;

		/* init userspace avc */
		sepgsql_avc_init();

		/* init privilege of the client */
		if (!MyProcPort)
		{
			/*
			 * SE-PgSQL does not prevent anything in single-user mode.
			 */
			sepostgresql_mode = SEPGSQL_MODE_INTERNAL;

			if (getprevcon_raw(&context) < 0)
				ereport(ERROR,
						(errcode(ERRCODE_INTERNAL_ERROR),
						 errmsg("SELinux: could not get server context")));
		}
		else
		{
			if (getpeercon_raw(MyProcPort->sock, &context) < 0)
				ereport(ERROR,
						(errcode(ERRCODE_INTERNAL_ERROR),
						 errmsg("SELinux: could not get client context")));
		}
		sepgsql_set_client_label(context);

		return;
	}
#endif
	if (default_with_secids)
	{
		default_with_secids = false;
		elog(LOG, "guc: default_with_secid was turned off "
			 "because no label based access control is availabel now");
    }
}

bool
sepgsql_worker_needed(void)
{
#ifdef HAVE_SELINUX
	if (sepgsql_is_enabled())
		return true;
#endif
	return false;
}

void
sepgsql_worker_main(void)
{
#ifdef HAVE_SELINUX
	if (sepgsql_is_enabled())
	{
		sepgsql_avc_worker_main();
		return;
	}
#endif
	elog(FATAL, "Bug? try to launch worker process without security provider");
}

Datum
sepgsql_getcon(PG_FUNCTION_ARGS)
{
#ifdef HAVE_SELINUX
	if (sepgsql_is_enabled())
	{
		char   *label = sepgsql_get_client_label();

		PG_RETURN_TEXT_P(cstring_to_text(label));
	}
#endif
	ereport(ERROR,
			(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
			 errmsg("SE-PostgreSQL is not available")));
	PG_RETURN_NULL();
}
