/*
 * pg_seclabel.h
 * Definition of the security label relation (pg_seclabel)
 *
 * Portions Copyright (c) 1996-2009, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 */
#ifndef PG_SECLABEL_H
#define PG_SECLABEL_H

#include "catalog/genbki.h"

#include "access/htup.h"
#include "access/skey.h"
#include "utils/relcache.h"

#define SecLabelRelationId		3400

CATALOG(pg_seclabel,3400) BKI_SHARED_RELATION BKI_WITHOUT_OIDS
{
	/* Identifier of the security label */
	Oid		secid;

	/* OID of the database which referes the entry */
	Oid		datid;

	/* OID of the table which refers the entry */
	Oid		relid;

	/* Text representation of the security label */
	text	label;
} FormData_pg_seclabel;

/*
 * Form_pg_seclabel corresponds to a pointer to a tuple with
 * the format of pg_seclabel relation.
 */
typedef FormData_pg_seclabel *Form_pg_seclabel;

/*
 * Compiler constants for pg_seclabel
 */
#define Natts_pg_seclabel			4
#define Anum_pg_seclabel_secid		1
#define Anum_pg_seclabel_datid		2
#define Anum_pg_seclabel_relid		3
#define Anum_pg_seclabel_label		4

/*
 * Functions to translate between security label and identifier
 */
extern bool ignore_security_label_input;

extern bool seclabelCatalogHasSysAttr(Oid relOid);
extern void seclabelPostBootstrap(void);
extern void seclabelOnCreateDatabase(Oid src_datOid, Oid dst_datOid);
extern void seclabelOnDropDatabase(Oid datOid);
extern void seclabelOnDropTable(Oid relOid);

extern Oid *seclabelMakeRelationDefaults(TupleDesc tupdesc, List *supOids);
extern Oid *seclabelMakeToastDefaults(TupleDesc tupdesc, Oid relOid);

extern Oid seclabelGetNewSecid(Relation rel, HeapTuple tuple);

extern Oid seclabelRawInput(Oid relOid, char *seclabel);
extern char *seclabelRawOutput(Oid relOid, Oid secid);
extern Oid seclabelTransInput(Oid relOid, char *seclabel);
extern char *seclabelTransOutput(Oid relOid, Oid secid);

extern Oid  seclabelMoveSecid(Oid dst_relid, Oid src_relid, Oid secid);
extern bool seclabelCompareSecid(Oid relid1, Oid secid1,
								 Oid relid2, Oid secid2);

extern Datum seclabelSysattOutput(Oid relOid, HeapTuple tuple);

extern void seclabelRelationReclaim(Oid relOid);
extern Datum seclabel_to_secid(PG_FUNCTION_ARGS);

#endif /* PG_SECLABEL_H */
