/*
 * sepgsql/sepgsql.h
 *
 * Header of SE-PostgreSQL Internal
 *
 * Copyright (C) 2006-2010, NEC Corporation
 *                          KaiGai Kohei <kaigai@ak.jp.nec.com>
 */
#ifndef SEPGSQL_H
#define SEPGSQL_H

#include "utils/snapshot.h"

/* GUC : sepostgresql */
extern int sepostgresql_mode;

#define SEPGSQL_MODE_DEFAULT		1
#define SEPGSQL_MODE_ENFORCING		2
#define SEPGSQL_MODE_PERMISSIVE		3
#define SEPGSQL_MODE_INTERNAL		4
#define SEPGSQL_MODE_DISABLED		5

/* GUC: sepostgresql_mcstrans */
extern bool sepgsql_mcstrans;

/* GUC: sepostgresql_debug_audit */
extern bool sepgsql_debug_audit;

/* Objject classes and permissions internally used */
enum SepgsqlClasses
{
	SEPG_CLASS_PROCESS = 0,
	SEPG_CLASS_FILE,
	SEPG_CLASS_DIR,
	SEPG_CLASS_LNK_FILE,
	SEPG_CLASS_CHR_FILE,
	SEPG_CLASS_BLK_FILE,
	SEPG_CLASS_SOCK_FILE,
	SEPG_CLASS_FIFO_FILE,
	SEPG_CLASS_DB_DATABASE,
	SEPG_CLASS_DB_SCHEMA,
	SEPG_CLASS_DB_TABLE,
	SEPG_CLASS_DB_VIEW,
	SEPG_CLASS_DB_SEQUENCE,
	SEPG_CLASS_DB_PROCEDURE,
	SEPG_CLASS_DB_COLUMN,
	SEPG_CLASS_DB_TUPLE,
	SEPG_CLASS_DB_BLOB,
	SEPG_CLASS_DB_LANGUAGE,
	SEPG_CLASS_MAX,
};

#define SEPG_PROCESS__TRANSITION			(1<<0)

#define SEPG_FILE__READ						(1<<0)
#define SEPG_FILE__WRITE					(1<<1)
#define SEPG_FILE__CREATE					(1<<2)
#define SEPG_FILE__GETATTR					(1<<3)
#define SEPG_FILE__UNLINK					(1<<4)
#define SEPG_FILE__RENAME					(1<<5)
#define SEPG_FILE__APPEND					(1<<6)

#define SEPG_DIR__READ						(SEPG_FILE__READ)
#define SEPG_DIR__WRITE						(SEPG_FILE__WRITE)
#define SEPG_DIR__CREATE					(SEPG_FILE__CREATE)
#define SEPG_DIR__GETATTR					(SEPG_FILE__GETATTR)
#define SEPG_DIR__UNLINK					(SEPG_FILE__UNLINK)
#define SEPG_DIR__RENAME					(SEPG_FILE__RENAME)
#define SEPG_DIR__SEARCH					(1<<6)
#define SEPG_DIR__ADD_NAME					(1<<7)
#define SEPG_DIR__REMOVE_NAME				(1<<8)
#define SEPG_DIR__RMDIR						(1<<9)
#define SEPG_DIR__REPARENT					(1<<10)

#define SEPG_LNK_FILE__READ					(SEPG_FILE__READ)
#define SEPG_LNK_FILE__WRITE				(SEPG_FILE__WRITE)
#define SEPG_LNK_FILE__CREATE				(SEPG_FILE__CREATE)
#define SEPG_LNK_FILE__GETATTR				(SEPG_FILE__GETATTR)
#define SEPG_LNK_FILE__UNLINK				(SEPG_FILE__UNLINK)
#define SEPG_LNK_FILE__RENAME				(SEPG_FILE__RENAME)

#define SEPG_CHR_FILE__READ					(SEPG_FILE__READ)
#define SEPG_CHR_FILE__WRITE				(SEPG_FILE__WRITE)
#define SEPG_CHR_FILE__CREATE				(SEPG_FILE__CREATE)
#define SEPG_CHR_FILE__GETATTR				(SEPG_FILE__GETATTR)
#define SEPG_CHR_FILE__UNLINK				(SEPG_FILE__UNLINK)
#define SEPG_CHR_FILE__RENAME				(SEPG_FILE__RENAME)

#define SEPG_BLK_FILE__READ					(SEPG_FILE__READ)
#define SEPG_BLK_FILE__WRITE				(SEPG_FILE__WRITE)
#define SEPG_BLK_FILE__CREATE				(SEPG_FILE__CREATE)
#define SEPG_BLK_FILE__GETATTR				(SEPG_FILE__GETATTR)
#define SEPG_BLK_FILE__UNLINK				(SEPG_FILE__UNLINK)
#define SEPG_BLK_FILE__RENAME				(SEPG_FILE__RENAME)

#define SEPG_SOCK_FILE__READ				(SEPG_FILE__READ)
#define SEPG_SOCK_FILE__WRITE				(SEPG_FILE__WRITE)
#define SEPG_SOCK_FILE__CREATE				(SEPG_FILE__CREATE)
#define SEPG_SOCK_FILE__GETATTR				(SEPG_FILE__GETATTR)
#define SEPG_SOCK_FILE__UNLINK				(SEPG_FILE__UNLINK)
#define SEPG_SOCK_FILE__RENAME				(SEPG_FILE__RENAME)

#define SEPG_FIFO_FILE__READ				(SEPG_FILE__READ)
#define SEPG_FIFO_FILE__WRITE				(SEPG_FILE__WRITE)
#define SEPG_FIFO_FILE__CREATE				(SEPG_FILE__CREATE)
#define SEPG_FIFO_FILE__GETATTR				(SEPG_FILE__GETATTR)
#define SEPG_FIFO_FILE__UNLINK				(SEPG_FILE__UNLINK)
#define SEPG_FIFO_FILE__RENAME				(SEPG_FILE__RENAME)

#define SEPG_DB_DATABASE__CREATE			(1<<0)
#define SEPG_DB_DATABASE__DROP				(1<<1)
#define SEPG_DB_DATABASE__GETATTR			(1<<2)
#define SEPG_DB_DATABASE__SETATTR			(1<<3)
#define SEPG_DB_DATABASE__RELABELFROM		(1<<4)
#define SEPG_DB_DATABASE__RELABELTO			(1<<5)
#define SEPG_DB_DATABASE__ACCESS			(1<<6)
#define SEPG_DB_DATABASE__LOAD_MODULE		(1<<7)

#define SEPG_DB_SCHEMA__CREATE				(SEPG_DB_DATABASE__CREATE)
#define SEPG_DB_SCHEMA__DROP				(SEPG_DB_DATABASE__DROP)
#define SEPG_DB_SCHEMA__GETATTR				(SEPG_DB_DATABASE__GETATTR)
#define SEPG_DB_SCHEMA__SETATTR				(SEPG_DB_DATABASE__SETATTR)
#define SEPG_DB_SCHEMA__RELABELFROM			(SEPG_DB_DATABASE__RELABELFROM)
#define SEPG_DB_SCHEMA__RELABELTO			(SEPG_DB_DATABASE__RELABELTO)
#define SEPG_DB_SCHEMA__SEARCH				(1<<6)
#define SEPG_DB_SCHEMA__ADD_NAME			(1<<7)
#define SEPG_DB_SCHEMA__REMOVE_NAME			(1<<8)

#define SEPG_DB_TABLE__CREATE				(SEPG_DB_DATABASE__CREATE)
#define SEPG_DB_TABLE__DROP					(SEPG_DB_DATABASE__DROP)
#define SEPG_DB_TABLE__GETATTR				(SEPG_DB_DATABASE__GETATTR)
#define SEPG_DB_TABLE__SETATTR				(SEPG_DB_DATABASE__SETATTR)
#define SEPG_DB_TABLE__RELABELFROM			(SEPG_DB_DATABASE__RELABELFROM)
#define SEPG_DB_TABLE__RELABELTO			(SEPG_DB_DATABASE__RELABELTO)
#define SEPG_DB_TABLE__SELECT				(1<<6)
#define SEPG_DB_TABLE__UPDATE				(1<<7)
#define SEPG_DB_TABLE__INSERT				(1<<8)
#define SEPG_DB_TABLE__DELETE				(1<<9)
#define SEPG_DB_TABLE__LOCK					(1<<10)
#define SEPG_DB_TABLE__INDEXON				(1<<11)

#define SEPG_DB_SEQUENCE__CREATE			(SEPG_DB_DATABASE__CREATE)
#define SEPG_DB_SEQUENCE__DROP				(SEPG_DB_DATABASE__DROP)
#define SEPG_DB_SEQUENCE__GETATTR			(SEPG_DB_DATABASE__GETATTR)
#define SEPG_DB_SEQUENCE__SETATTR			(SEPG_DB_DATABASE__SETATTR)
#define SEPG_DB_SEQUENCE__RELABELFROM		(SEPG_DB_DATABASE__RELABELFROM)
#define SEPG_DB_SEQUENCE__RELABELTO			(SEPG_DB_DATABASE__RELABELTO)
#define SEPG_DB_SEQUENCE__GET_VALUE			(1<<6)
#define SEPG_DB_SEQUENCE__NEXT_VALUE		(1<<7)
#define SEPG_DB_SEQUENCE__SET_VALUE			(1<<8)

#define SEPG_DB_VIEW__CREATE				(SEPG_DB_DATABASE__CREATE)
#define SEPG_DB_VIEW__DROP					(SEPG_DB_DATABASE__DROP)
#define SEPG_DB_VIEW__GETATTR				(SEPG_DB_DATABASE__GETATTR)
#define SEPG_DB_VIEW__SETATTR				(SEPG_DB_DATABASE__SETATTR)
#define SEPG_DB_VIEW__RELABELFROM			(SEPG_DB_DATABASE__RELABELFROM)
#define SEPG_DB_VIEW__RELABELTO				(SEPG_DB_DATABASE__RELABELTO)
#define SEPG_DB_VIEW__EXPAND				(1<<6)

#define SEPG_DB_PROCEDURE__CREATE			(SEPG_DB_DATABASE__CREATE)
#define SEPG_DB_PROCEDURE__DROP				(SEPG_DB_DATABASE__DROP)
#define SEPG_DB_PROCEDURE__GETATTR			(SEPG_DB_DATABASE__GETATTR)
#define SEPG_DB_PROCEDURE__SETATTR			(SEPG_DB_DATABASE__SETATTR)
#define SEPG_DB_PROCEDURE__RELABELFROM		(SEPG_DB_DATABASE__RELABELFROM)
#define SEPG_DB_PROCEDURE__RELABELTO		(SEPG_DB_DATABASE__RELABELTO)
#define SEPG_DB_PROCEDURE__EXECUTE			(1<<6)
#define SEPG_DB_PROCEDURE__ENTRYPOINT		(1<<7)
#define SEPG_DB_PROCEDURE__INSTALL			(1<<8)

#define SEPG_DB_COLUMN__CREATE				(SEPG_DB_DATABASE__CREATE)
#define SEPG_DB_COLUMN__DROP				(SEPG_DB_DATABASE__DROP)
#define SEPG_DB_COLUMN__GETATTR				(SEPG_DB_DATABASE__GETATTR)
#define SEPG_DB_COLUMN__SETATTR				(SEPG_DB_DATABASE__SETATTR)
#define SEPG_DB_COLUMN__RELABELFROM			(SEPG_DB_DATABASE__RELABELFROM)
#define SEPG_DB_COLUMN__RELABELTO			(SEPG_DB_DATABASE__RELABELTO)
#define SEPG_DB_COLUMN__SELECT				(1<<6)
#define SEPG_DB_COLUMN__UPDATE				(1<<7)
#define SEPG_DB_COLUMN__INSERT				(1<<8)

#define SEPG_DB_TUPLE__RELABELFROM			(SEPG_DB_DATABASE__RELABELFROM)
#define SEPG_DB_TUPLE__RELABELTO			(SEPG_DB_DATABASE__RELABELTO)
#define SEPG_DB_TUPLE__SELECT				(SEPG_DB_DATABASE__GETATTR)
#define SEPG_DB_TUPLE__UPDATE				(SEPG_DB_DATABASE__SETATTR)
#define SEPG_DB_TUPLE__INSERT				(SEPG_DB_DATABASE__CREATE)
#define SEPG_DB_TUPLE__DELETE				(SEPG_DB_DATABASE__DROP)

#define SEPG_DB_BLOB__CREATE				(SEPG_DB_DATABASE__CREATE)
#define SEPG_DB_BLOB__DROP					(SEPG_DB_DATABASE__DROP)
#define SEPG_DB_BLOB__GETATTR				(SEPG_DB_DATABASE__GETATTR)
#define SEPG_DB_BLOB__SETATTR				(SEPG_DB_DATABASE__SETATTR)
#define SEPG_DB_BLOB__RELABELFROM			(SEPG_DB_DATABASE__RELABELFROM)
#define SEPG_DB_BLOB__RELABELTO				(SEPG_DB_DATABASE__RELABELTO)
#define SEPG_DB_BLOB__READ					(1<<6)
#define SEPG_DB_BLOB__WRITE					(1<<7)
#define SEPG_DB_BLOB__IMPORT				(1<<8)
#define SEPG_DB_BLOB__EXPORT				(1<<9)

#define SEPG_DB_LANGUAGE__CREATE			(SEPG_DB_DATABASE__CREATE)
#define SEPG_DB_LANGUAGE__DROP				(SEPG_DB_DATABASE__DROP)
#define SEPG_DB_LANGUAGE__GETATTR			(SEPG_DB_DATABASE__GETATTR)
#define SEPG_DB_LANGUAGE__SETATTR			(SEPG_DB_DATABASE__SETATTR)
#define SEPG_DB_LANGUAGE__RELABELFROM		(SEPG_DB_DATABASE__RELABELFROM)
#define SEPG_DB_LANGUAGE__RELABELTO			(SEPG_DB_DATABASE__RELABELTO)
#define SEPG_DB_LANGUAGE__IMPLEMENTE		(1<<6)
#define SEPG_DB_LANGUAGE__EXECUTE			(1<<7)

/*
 * sepgsql_sid_t : alternative representation of security context
 */
typedef struct {
	Oid		relid;
	Oid		secid;
} sepgsql_sid_t;

struct av_decision;

/*
 * selinux.c
 */
extern bool sepgsql_is_enabled(void);
extern bool sepgsql_get_enforce(void);
extern const char *sepgsql_show_mode(void);

extern Size sepgsql_shmem_size(void);

extern void sepgsql_audit_log(bool denied,
							  char *scontext,
							  char *tcontext,
							  uint16 tclass,
							  uint32 audited,
							  const char *audit_name);
extern void sepgsql_compute_avd(char *scontext,
								char *tcontext,
								uint16 tclass,
								struct av_decision *avd);
extern bool sepgsql_compute_perms(char *scontext,
								  char *tcontext,
								  uint16 tclass,
								  uint32 required,
								  const char *audit_name,
								  bool abort);
extern char *sepgsql_compute_create(char *scontext,
									char *tcontext,
									uint16 tclass);
extern bool sepgsql_client_perms(sepgsql_sid_t tsid,
								 uint16 tclass,
								 uint32 required,
								 const char *audit_name,
								 bool abort);
extern sepgsql_sid_t sepgsql_client_create_secid(sepgsql_sid_t tsid,
												 uint16 tclass,
												 Oid nrelid);
extern char *sepgsql_client_create_label(sepgsql_sid_t tsid,
										 uint16 tclass);
extern void sepgsql_avc_worker_main(void);

/*
 * avc.c
 */
extern Size sepgsql_shmem_size(void);
extern void sepgsql_avc_init(void);
extern void sepgsql_avc_switch(const char *scontext);


/*
 * label.c
 */
extern sepgsql_sid_t sepgsql_move_secid(Oid dst_relid, sepgsql_sid_t ssid);

extern sepgsql_sid_t sepgsql_get_default_database_secid(Oid templateOid);
extern sepgsql_sid_t sepgsql_get_default_schema_secid(Oid databaseOid);
extern sepgsql_sid_t sepgsql_get_default_table_secid(Oid namespaceOid);
extern sepgsql_sid_t sepgsql_get_default_sequence_secid(Oid namespaceOid);
extern sepgsql_sid_t sepgsql_get_default_view_secid(Oid namespaceOid);
extern sepgsql_sid_t sepgsql_get_default_proc_secid(Oid namespaceOid);
extern sepgsql_sid_t sepgsql_get_default_column_secid(Oid tableOid);
extern sepgsql_sid_t sepgsql_get_default_tuple_secid(Oid tableOid);
extern sepgsql_sid_t sepgsql_get_default_blob_secid(Oid databaseOid);
extern Oid sepgsql_get_default_secid(Relation rel, HeapTuple tuple);

extern void sepgsql_initial_labeling(void);

extern char *sepgsql_mcstrans_out(char *label);
extern char *sepgsql_mcstrans_in(char *label);
extern char *sepgsql_rawlabel_out(char *label);
extern char *sepgsql_rawlabel_in(char *label);

/*
 * sepgsql_(object)_common
 */
extern bool sepgsql_database_common(Oid datOid, uint32 required, bool abort);
extern bool sepgsql_schema_common(Oid nspOid, uint32 required, bool abort);
extern bool sepgsql_relation_common(Oid relOid, uint32 required, bool abort);
extern bool sepgsql_attribute_common(Oid relOid, AttrNumber attno,
									 uint32 required, bool abort);
extern bool sepgsql_proc_common(Oid procOid, uint32 required, bool abort);
extern bool sepgsql_type_common(Oid typeOid, uint32 required, bool abort);
extern bool sepgsql_cast_common(Oid srcTypeOid, Oid dstTypeOid,
								uint32 required, bool abort);
extern bool sepgsql_tablespace_common(Oid tspaceOid, uint32 required, bool abort);
extern bool sepgsql_operator_common(Oid operOid, uint32 required, bool abort);
extern bool sepgsql_opclass_common(Oid opcOid, uint32 required, bool abort);
extern bool sepgsql_opfamily_common(Oid opfOid, uint32 required, bool abort);
extern bool sepgsql_role_common(Oid roleOid, uint32 required, bool abort);
extern bool sepgsql_largeobejct_common(Oid loid, Snapshot snapshot,
									   uint32 required, bool abort);
extern bool sepgsql_conversion_common(Oid convOid, uint32 required, bool abort);
extern bool sepgsql_largeobject_common(Oid loid, Snapshot snapshot,
									   uint32 required, bool abort);
extern bool sepgsql_ts_config_common(Oid confOid, uint32 required, bool abort);
extern bool sepgsql_ts_dict_common(Oid dictOid, uint32 required, bool abort);
extern bool sepgsql_ts_parser_common(Oid parseOid, uint32 required, bool abort);
extern bool sepgsql_ts_template_common(Oid templateOid, uint32 required, bool abort);
extern bool sepgsql_fdw_common(Oid fdwOid, uint32 required, bool abort);
extern bool sepgsql_fserver_common(Oid fservOid, uint32 required, bool abort);
extern bool sepgsql_user_mapping_common(Oid umapOid, uint32 required, bool abort);

#endif	/* SEPGSQL_H */
